==> piippin-0.1.5.jar: piippin/core.clj <==
(ns piippin.core
  (:require [clojure.string :as str]
            [slingshot.slingshot :refer [throw+]]))

(def dob (str "(?i)(dob|birth).{0,20}"
              "(\\d{2,4}-\\d{2}-\\d{2}|\\d{2}/\\d{2}/\\d{2,4})"))

(def common-patterns
  {"longnum"    (re-pattern "\\d{6,}")
   ;https://www.regular-expressions.info/creditcard.html
   "creditcard" (re-pattern "(?:\\d[ -]*?){13,16}")
   ;http://www.cardinalpath.com/what-you-need-to-know-about-google-analytics-\
   ;personally-identifiable-information/
   "email"      (re-pattern (str "([a-zA-Z0-9_\\.-]+)@([\\da-zA-Z\\.-]+)"
                                 ".([a-zA-Z\\.]{2,6})"))
   "ssn"        (re-pattern "\\d{3}-?\\d{2}-?\\d{4}")
   ;https://stackoverflow.com/questions/16699007/regular-expression-to-match-\
   ;standard-10-digit-phone-number
   "phone"      (re-pattern (str "(\\+\\d{1,2}[ \\t])?\\(?\\d{3}\\)?[[ \\t].-]?"
                                 "\\d{3}[ \\t.-]?\\d{4}"))
   "dob"        (re-pattern dob)})


(defn- simpleid []
  (str/join (map char (take 10 (repeatedly (comp (partial + 97)
                                                 (partial rand-int 26)))))))

(defn replace-with-fingerprint [s replacements]
  {:result       (reduce-kv (fn [s k v] (str/replace s k v)) s replacements)
   :replacements replacements})

(defn- count-groups [match]
  (if (vector? match) (count match) 0))

(defn fingerprint [exceptions s]
  (let [found
        (remove (comp nil? second)
                (map #(vector % (re-seq % s)) exceptions))
        results (map (fn [[p matches]]
                       (let [match-counts (map count-groups matches)]
                         [p (apply min match-counts)])) found)
        bad-matches (remove (comp (partial = 2) second) results)]
    (if (empty? bad-matches)
      (replace-with-fingerprint
        s
        (zipmap (map second (reduce concat (map second found)))
                (repeatedly simpleid)))
      (do
        (throw+ {:type ::bad-expression}
                (str "Regular expression(s) must have exactly one group: "
                     (str/join ", " (map (comp (partial format "'%s'")
                                               first) bad-matches))))
        s))))

(defn return [replacements fingerprinted-string]
  (reduce-kv (fn [s k v] (str/replace s v k))
             fingerprinted-string replacements))

(defn find-pii
  "Given a type `t`, check if the string `s` contains any lines that match
   either the regular expression that is found in `common-patterns` with
   its name or a regular expression that contains the value itself."
  [exceptions t s]
  (let [{:keys [result replacements]} (fingerprint exceptions s)
        lines (str/split-lines result)

        pattern (common-patterns t)
        pattern (or pattern (re-pattern (str "(?i)" t)))]
    (->> lines
         (filter (partial re-find pattern))
         (map (partial return replacements)))))

(defn remove-pii
  "Remove any PII from the given `s`"
  [exceptions s]
  (reduce (fn [s p]
            (let [{:keys [result replacements]} (fingerprint exceptions s)
                  masked (str/replace result p "****")]
              (return replacements masked)))
          (or s "")
          (vals common-patterns)))

(defn scrub-pii-middleware [excluded-formats exceptions]
  (fn [{:keys [vargs ?msg-fmt ?err] :as r}]
    (let [scrubbed-emsg (when ?err (remove-pii exceptions (.getMessage ?err)))
          ?err (when scrubbed-emsg (proxy [Throwable] [scrubbed-emsg]
                                     (getMessage [] scrubbed-emsg)
                                     (getStackTrace [] (.getStackTrace ?err))
                                     (getCause [] (.getCause ?err))))]
      (if-not (excluded-formats ?msg-fmt)
        (assoc r :vargs (map (fn [varg] (remove-pii exceptions (str varg)))
                             vargs)
                 :?err ?err)
        r))))

==> piippin-0.1.6.jar: piippin/core.clj <==
(ns piippin.core
  (:require [clojure.string :as str]
            [slingshot.slingshot :refer [throw+]]))

(def dob (str "(?i)(dob|birth).{0,20}"
              "(\\d{2,4}-\\d{2}-\\d{2}|\\d{2}/\\d{2}/\\d{2,4})"))

(def common-patterns
  {"longnum"    (re-pattern "\\d{6,}")
   ;https://www.regular-expressions.info/creditcard.html
   "creditcard" (re-pattern "(?:\\d[ -]*?){13,16}")
   ;http://www.cardinalpath.com/what-you-need-to-know-about-google-analytics-\
   ;personally-identifiable-information/
   "email"      (re-pattern (str "([a-zA-Z0-9_\\.-]+)@([\\da-zA-Z\\.-]+)"
                                 ".([a-zA-Z\\.]{2,6})"))
   "ssn"        (re-pattern "\\d{3}-?\\d{2}-?\\d{4}")
   ;https://stackoverflow.com/questions/16699007/regular-expression-to-match-\
   ;standard-10-digit-phone-number
   "phone"      (re-pattern (str "(\\+\\d{1,2}[ \\t])?\\(?\\d{3}\\)?[[ \\t].-]?"
                                 "\\d{3}[ \\t.-]?\\d{4}"))
   "dob"        (re-pattern dob)})


(defn- simpleid []
  (str/join (map char (take 10 (repeatedly (comp (partial + 97)
                                                 (partial rand-int 26)))))))

(defn replace-with-fingerprint [s replacements]
  {:result       (reduce-kv (fn [s k v] (str/replace s k v)) s replacements)
   :replacements replacements})

(defn fingerprint [exclusions s]
  (let [found
        (remove (comp nil? first)
                (map #(vector % (re-seq % s)) exclusions))]
    (replace-with-fingerprint
      s
      (zipmap (map second (reduce concat (map second found)))
              (repeatedly simpleid)))))

(defn return [replacements fingerprinted-string]
  (reduce-kv (fn [s k v] (str/replace s v k))
             fingerprinted-string replacements))

(defn find-pii
  "Given a type `t`, check if the string `s` contains any lines that match
   either the regular expression that is found in `common-patterns` with
   its name or a regular expression that contains the value itself."
  [exceptions t s]
  (let [{:keys [result replacements]} (fingerprint exceptions s)
        lines (str/split-lines result)

        pattern (common-patterns t)
        pattern (or pattern (re-pattern (str "(?i)" t)))]
    (->> lines
         (filter (partial re-find pattern))
         (map (partial return replacements)))))

(defn remove-pii
  "Remove any PII from the given `s`"
  [exceptions s]
  (reduce (fn [s p]
            (let [{:keys [result replacements]} (fingerprint exceptions s)
                  masked (str/replace result p "****")]
              (return replacements masked)))
          (or s "")
          (vals common-patterns)))

(defn scrub-pii-middleware [excluded-formats exceptions]
  (fn [{:keys [vargs ?msg-fmt ?err] :as r}]
    (let [scrubbed-emsg (when ?err (remove-pii exceptions (.getMessage ?err)))
          ?err (when scrubbed-emsg (proxy [Throwable] [scrubbed-emsg]
                                     (getMessage [] scrubbed-emsg)
                                     (getStackTrace [] (.getStackTrace ?err))
                                     (getCause [] (.getCause ?err))))]
      (if-not (excluded-formats ?msg-fmt)
        (assoc r :vargs (map (fn [varg] (remove-pii exceptions (str varg)))
                             vargs)
                 :?err ?err)
        r))))
